/*
 
 Copyright (c) 2004 PXI Project Team
 
 Permission is hereby granted, free of charge, to any person obtaining a copy of
 this software and associated documentation files (the "Software"), to deal in the
 Software without restriction, including without limitation the rights to use,
 copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
 Software, and to permit persons to whom the Software is furnished to do so,
 subject to the following conditions:
 
 The above copyright notice and this permission notice shall be included in all copies
 or substantial portions of the Software.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 */

/*

       __   ___  / /
   ___( (\_( ) )/ /
  |\___` \\_\/_/_/_\
  | | ____/\_\`._._...^^^^^^^^^^^^^^^^^^\
  | ||\__/ '_\ \`_._\.;    \_\ |\\)\)\|  \
   \|| | | | | |    | \     \ \|-\\ \ |   \
    || | |^| | |    |  \       __          \
     \ |     | |    |   \ \)\\) \ \_\ |\|\\|\
      \|ejm__| |____|    \ \)\\\ \ \ \|/|-\| \
                         ^^^^^^^^^^^^^^^^^^^^^^

Happy 20th Birthday Tyler Vano    :)     6/8/04

*/


#ifndef _____BNF_GRAMMAR_____
#define _____BNF_GRAMMAR_____

#include <string>
#include <vector>
using namespace std;

const int BNF_GF_TYPE_LITERAL = 0;
const int BNF_GF_TYPE_RULE_REF = 1;
const int BNF_GF_TYPE_BUILT_IN = 2;
const int BNF_GF_TYPE_MULTIPLE = 3;
const int BNF_GF_TYPE_SEQUENCE = 3;
const int BNF_GF_TYPE_SUBFACTOR = 4;
const int BNF_GF_TYPE_CHOICE = 4;

const int BNF_GR_TYPE_DEFINITION = 0;
const int BNF_GR_TYPE_PRAGMA = 1;

const int BNF_DP_TYPE_NORMAL = 0;
const int BNF_DP_TYPE_MULTIPLE = 1;
const int BNF_DP_TYPE_OPTIONAL = 2;
const int BNF_DP_TYPE_SINGULAR = 0;
const int BNF_DP_TYPE_ZERO_OR_MORE = 1;
const int BNF_DP_TYPE_ONE_OR_MORE = 2;

string bnf_reformat(const string &);
string bnf_decode_esc_str(const string &);
string bnf_encode_esc_str(const string &);

class grammarFactor
{
	public:

	int type;
	int duplicity;
	bool optional;
	string value;
	string compilerHandler;
	vector<grammarFactor> factors;

	void clear(void);
	void readData(vector<string> &, vector<string>::iterator &);
	string writeData(void);
	void dump(int);
	
	string stype(void);
	string dtype(void);
};

class grammarRule
{
	public:

	int type;
	string name;
	string pragmaDef;
	vector<grammarFactor> configurations;

	void clear(void);
	void readData(const string &);
	string writeData(void);
	void dump(void);

	string stype(void);
};

class grammarFile
{
	public:

	string languageName;
	string grammarInfo;
	vector<grammarRule> rules;
	string script;

	void clear(void);
	void readFile(const string &);
	void readData(const string &);
	void writeFile(const string &);
	string writeData(void);
	void dump(void);
	
	string getPragma( const string & );
	grammarRule & getRule( int, const string & );
	
};

#endif